Include "mri2D.inc"
Include "gui.pro"
Include "gui-gmsh.pro"
//Include "fieldOnLegs.pro"

Group{
  Body=Region[{1,2}] ;
  Jcur=Region[1] ;
  bnd = Region[4];
  For itheta In {0:Ntheta-1}
    Antenna~{itheta}=Region[(1000+itheta)];
  EndFor
  Antennas = Region[(1000:1000+Ntheta-1)];
  Air = Region[{3,Antennas,bnd}] ;
  Domain = Region[{Body,Air,Jcur,Antennas,bnd}] ;
  Lossy= Region[{Body}];
}

Function{
  f = 0.3e9;
  c=3e8;
  nu0=1/(4.0e-7*Pi);
  epsilon0=8.84e-12;
  pisur2RM=Atan[1.0]*2.0/RM;
  R[]=Sqrt[(X[]-x0)^2+(Y[]-y0)^2];
  M[]=((X[]-x0)-(Y[]-y0)*Complex[0,1])*pisur2RM*Cos[R[]*pisur2RM];
  
  Bp[]=Vector[-1.0,Complex[0,-1.0],0.0];
  Bm[]=Vector[-1.0,-Complex[0,-1.0],0.0];
  epsilon[Air]=epsilon0;
  epsilon[Lossy]=epsilon0*epsilonHead;
  sourceTR = ListFromFile["fieldOnLegs.pos"];
  sigma[Lossy]=sigmaHead;
  If (TXRX == 0)
    Coefjs0 = 1.0;
    Coefjs= 0.0;
    Printf["TRANSMIT"];
  Else
    Coefjs0 = 0.0;
    Coefjs = 1.0;
  EndIf
  
  js0[Jcur]=M[]*Sqrt[-1]*Coefjs0;
  
  For itheta In {0:Ntheta-1}
    phase=itheta/Ntheta*2.0*Pi;
    If (CIRCORTR==0)
        js[Antenna~{itheta}]=Complex[Cos[phase],Sin[phase]]*Coefjs;
    ElseIf (CIRCORTR==1)
        js[Antenna~{itheta}]=Complex[sourceTR(3*itheta+2),-sourceTR(3*itheta+3)]*Coefjs;
    Else
       angleSignal = Atan2[-sourceTR(3*itheta+3),sourceTR(3*itheta+2)];
       js[Antenna~{itheta}]=Complex[Cos[angleSignal],Sin[angleSignal]]*Coefjs;
    EndIf
    
  EndFor
  Printf["PEC = %g , TXRX = %g , CIRCORTR = %g ",PEC,TXRX,CIRCORTR];
  
}



Jacobian{
{ Name JSur ; Case { { Region All ; Jacobian Sur ; } } }
{ Name JVol ; Case { { Region All ; Jacobian Vol ; } } }
}
Integration {
  { Name I1 ;
    Case {
      { Type Gauss ;
        Case {
	  { GeoElement Point       ; NumberOfPoints  1 ; }
	  { GeoElement Line        ; NumberOfPoints  3 ; }
	  { GeoElement Triangle    ; NumberOfPoints  4 ; }
	  { GeoElement Quadrangle  ; NumberOfPoints  4 ; }
	  { GeoElement Tetrahedron ; NumberOfPoints  4 ; }
	  { GeoElement Hexahedron  ; NumberOfPoints  6 ; }
	  { GeoElement Prism       ; NumberOfPoints  6 ; }
	}
      }
    }
  }

  { Name I2 ;
    Case {
      { Type Gauss ;
        Case {
	  { GeoElement Point       ; NumberOfPoints  1 ; }
	  { GeoElement Line        ; NumberOfPoints  4 ; }
	  { GeoElement Triangle    ; NumberOfPoints  7 ; }
	  { GeoElement Quadrangle  ; NumberOfPoints  7 ; }
	  { GeoElement Tetrahedron ; NumberOfPoints  15 ; }
	  { GeoElement Hexahedron  ; NumberOfPoints  34 ; }
	  { GeoElement Prism       ; NumberOfPoints  21 ; }
	}
      }
    }
  }
  { Name I3 ;
    Case {
      { Type Gauss ;
        Case {
          { GeoElement Point ; NumberOfPoints  1 ; }
          { GeoElement Line ; NumberOfPoints  4 ; }
          { GeoElement Triangle ; NumberOfPoints  6 ; }
          { GeoElement Quadrangle ; NumberOfPoints 7 ; }
          { GeoElement Tetrahedron ; NumberOfPoints 15 ; }
          { GeoElement Hexahedron ; NumberOfPoints 34 ; }
        }
      }
    }
  }
}

Constraint {
	{ Name ElectricField; Type Assign;
	  Case { {Region bnd; Value 0.0;}}
	}
}
FunctionSpace {
  //Electric field
  { Name Scalar_ez; Type Form0;
    BasisFunction {
      { Name se; NameOfCoef ee; Function BF_Node;
        Support Region[Domain] ; Entity NodesOf[All]; }
    }
    If (PEC==1)
    Constraint {
      { NameOfCoef ee; EntityType NodesOf ; NameOfConstraint ElectricField; }
    }
    EndIf
  }



}

Formulation {
 { Name Microwave_e ; Type FemEquation;
    Quantity {
      { Name e; Type Local;  NameOfSpace Scalar_ez; }
    }
    Equation {
      Galerkin { [ nu0 * Dof{Grad e} , {Grad e} ];
        In Domain; Integration I3; Jacobian JVol;  }
      
      Galerkin { DtDof [ sigma[] * Dof{e} , {e} ];
       In Lossy ; Integration I3; Jacobian JVol;  }
      Galerkin { DtDtDof [ epsilon[] * Dof{e} , {e} ];
        In Domain; Integration I3; Jacobian JVol;  }

      Galerkin { DtDof [  js0[] , {e} ];
        In Jcur; Integration I3; Jacobian JVol;  }

      Galerkin { DtDof [  js[] , {e} ];
        In Antennas; Integration I3; Jacobian JVol;  }

    
	//Sommerfeld radiation condition (approx.)
    If  (PEC==0)
	Galerkin{DtDof[nu0/c*Dof{e}, {e}];
	  In bnd; Jacobian JSur; Integration I3;}
	EndIf

     
    }
  }
}


Resolution {
  { Name Microwave_e_BC;
    System {
      { Name B; 
        NameOfFormulation Microwave_e; 
        Type ComplexValue;
        Frequency f;}
    }
    Operation {
      Generate[B]; Solve[B]; SaveSolution[B];
    }
  }
}

PostProcessing {
  { Name Microwave_e ; NameOfFormulation Microwave_e ;
    Quantity 
    {
       { Name Ez  ;  
         Value { Local{ [{e}] ; In Domain ; Jacobian JSur;} } 
       }
       { Name Hp  ;  
        Value { 
          Local{ [ Abs[Bp[]*{d e}]] ; In Domain ; Jacobian JSur;} 
              }
       }
       { Name Hm  ;  
        Value { 
          Local{ [ Abs[Bm[]*{d e}]] ; In Domain ; Jacobian JSur;} } 
       }
     }
  }
}


PostOperation {

  { Name Microwave_e ; NameOfPostProcessing Microwave_e ;
    Operation {
      Print[ Ez,  OnElementsOf Region[{Domain}],  File "toto.pos" ] ;    
      Print[ Hp ,  OnElementsOf Region[{Lossy}],  File "totoH.pos" ] ;    
      Print[ Hm ,  OnElementsOf Region[{Lossy}],  File "totoH.pos" ] ;
         If (TXRX == 0)
		Print[ Ez, OnGrid {Rprobe * Cos[$A], Rprobe * Sin[$A], 0}
        {0:2*Pi:2*Pi/Ntheta, 0, 0},File "fieldOnLegs.pos",Format ElementTable];
        EndIf
     }
  }

}

